using System;
using System.IO;
using System.Reflection;
using System.Text;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.ComponentModel;
using System.Globalization;
namespace Andrei.CommonControlsWeb
{
	/// <summary>
	/// Control to display as many RSS Reader's 
	/// for a given RSS ( the property's name is URL)
	/// </summary>
	[DefaultProperty("Url")]
	[ToolboxData("<{0}:AddRss runat=server></{0}:AddRss>")]
	[ParseChildren(true)]
	public class AddRSS : System.Web.UI.WebControls.WebControl,INamingContainer
	{
		/// <summary>
		/// Name for the RSS
		/// </summary>
		[Bindable(true)]
		[Category("Appearance")]
		[DefaultValue("")]
		public string Title
		{
			get
			{
				string s = (string)ViewState["Title"];
				return (s == null) ? "Add my RSS" : s;
			}
			set
			{

				ViewState["Title"] = value;
			}
		}
		
		
		/// <summary>
		/// The RSS Adress
		/// </summary>
		[Bindable(true)]
		[Category("Appearance")]
		[DefaultValue("")]
		public string Url
		{
			get
			{
				string s = (string)ViewState["Url"];
				return (s == null) ? String.Empty : s;
			}
			set
			{

				ViewState["Url"] = value;
				//reset encoded url - for regenerating
				m_UrlEncoded = null;
				CreateChildControls();
				
			}
		}

		/// <summary>
		/// path where to save images
		/// it is usually better to unload images from project
		/// and to put directly to the web server.
		/// </summary>
		public string PathToSaveImages
		{
			set
			{
				ViewState["PathToSaveImages"] = value;
				if (this.Page == null || this.Page.Server == null)
					return;
				if (value == null || value.Length == 0)
					return;

				string strPath = this.Page.Server.MapPath(value);
				
				Assembly a = Assembly.GetExecutingAssembly();
				foreach (string strRes in a.GetManifestResourceNames())
				{

					//TODO : find a better way to remove the prefix AddRss.Images
					string strPathFile = Path.Combine(strPath,strRes.Replace("AddRss_Obj.Images.",""));

					if (File.Exists(strPathFile))
						continue;

					Stream s= a.GetManifestResourceStream(strRes);
					System.Drawing.Image.FromStream(s).Save(strPathFile);
				}
				
			}
			get
			{
				string s = (string)ViewState["PathToSaveImages"];
				return (s == null) ? "." : s;
			}
			
		}
		/// <summary>
		/// encoded RSS url
		/// </summary>
		 internal string UrlEncoded
		{
			get
			{
				if (m_UrlEncoded == null)
				{
					if (this.Page != null && this.Page.Server != null)
					{
						m_UrlEncoded = this.Page.Server.UrlEncode(Url);
					}
				}
				return m_UrlEncoded;
			}
		} 
		/// <summary>
		/// private variable to hold the encoded path to RSS
		/// </summary>
		private string m_UrlEncoded = null;
		
		/// <summary>
		/// can be _blank, etc
		/// </summary>
		[Bindable(true)]
		[Category("Appearance")]
		[DefaultValue("_blank")]
		public string Target
		{
			get
			{
				string s = (string)ViewState["Target"];
				return (s == null) ? "_blank" : s;
			}
			set
			{

				ViewState["Target"] = value;
			}
		}

		/// <summary>
		/// template for customizing the control
		/// </summary>
		[Browsable(false)]
		[PersistenceMode(PersistenceMode.InnerProperty)]
		[TemplateContainer(typeof(AddRssContainer))]
		public ITemplate RSSTemplate
		{

			get { return m_RSSTemplate; }

			set { m_RSSTemplate = value; }

		} 
		/// <summary>
		/// private variable to hold the template
		/// </summary>
		private ITemplate m_RSSTemplate;

		
		/// <summary>
		/// override this method to ensure (at least)default template
		/// </summary>
		public override void DataBind()
		{
			EnsureChildControls();
			base.DataBind();
		}
		/// <summary>
		/// create a new instannce 
		/// of the template if the developer does not provide one
		/// </summary>
		protected override void CreateChildControls()
		{
			Controls.Clear();
			PathToSaveImages = PathToSaveImages;
			if (RSSTemplate == null)
			{
				RSSTemplate = new AddRssDefaultTemplate();
			}

			AddRssContainer m_RSSContainer = new AddRssContainer(this);

			RSSTemplate.InstantiateIn(m_RSSContainer);
			Controls.Add(m_RSSContainer);			
		}
		private string ConstructRSS(string Prefix, string ImageSrc,bool EncodeUrl)
		{
			return "<a href='" + Prefix +((EncodeUrl)?UrlEncoded : Url) + "' target='" + Target + "'>" +
				"<img src=" + Path.Combine(PathToSaveImages, ImageSrc) + " border=0>" +
				"</a>";
		}
		/// <summary>
		/// construct default RSS from URL and Image
		/// </summary>
		/// <param name="Prefix">prefix for adding the RSS URL</param>
		/// <param name="ImageSrc">source of the image</param>
		/// <returns>string to display the RSS</returns>
		private string ConstructRSS(string Prefix, string ImageSrc)
		{
			return ConstructRSS(Prefix, ImageSrc, true);
		}


		#region RSS To Display
		public string OriginalRSS
		{
			get
			{
				return ConstructRSS("", "xml.gif",false);
			}
		}

		public string YahooRSSDisplay
		{
			get
			{
				return ConstructRSS("http://add.my.yahoo.com/rss?url=", "addtomyyahoo.GIF");
			}

		}
		public string NewsGatorRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.newsgator.com/ngs/subscriber/subext.aspx?url=", "ngsub1.gif");
			}
		}
		public string FeedReaderRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.feedreader.net/rss/rss.cgi?feedsrc=", "RSS_FeedReaderLogo091bp.gif");
			}

		}
		public string RocketRSSDisplay
		{
			get
			{
				return ConstructRSS("http://reader.rocketinfo.com/desktop/rocketreader.jsp?feed=", "tryrocket.png");
			}
		}
		public string BotABlogRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.botablog.com/botthisblog.php?blog=", "botablog.gif");
			}

		}

		public string BlogLinesRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.bloglines.com/sub/", "bloglines.gif");
			}
		}

		public string MSNRSSDisplay
		{
			get
			{
				return ConstructRSS("http://my.msn.com/addtomymsn.armx?id=rss&ut=", "MyMSN.gif");
			}
		}

		public string PluckRSSDisplay
		{
			get
			{
				return ConstructRSS("http://feedfinder.pluck.com/PreviewFeed.aspx?uri=", "plusPluck.gif");
			}
		}

		public string NewsIsFreeRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.newsisfree.com/user/sub/?url=", "sub_nif4.gif");
			}
		}

		public string FeedsterRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.feedster.com/myfeedster.php?action=addrss&rssurl=", "feedster.png");
			}
		}

		public string FyuzeRSSDisplay
		{
			get
			{
				return ConstructRSS("http://fyuze.com/add/rss/", "fyuze.gif");
			}
		}

		public string RojoRSSDisplay
		{
			get
			{
				return ConstructRSS("http://www.rojo.com/add-subscription?resource=", "RojoWideRed.gif");
			}
		}
		#endregion
		
	}
}
